"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultDropExtensions = exports["default"] = void 0;

var nodePath = require('path');

var defaultDropExtensions = ['.spec', '.test', '.story', '.stories'];
exports.defaultDropExtensions = defaultDropExtensions;

var normalisePathSep = function normalisePathSep(path) {
  return path ? path.replace(/\\/g, '/') : path;
};

var removeLeadingDotSlash = function removeLeadingDotSlash(path) {
  return path && path.match(/^\.\//) ? path.replace(/^\.\//, '') : path;
};

var buildPluginConfig = function buildPluginConfig(opts) {
  return {
    dropAllFilenames: opts && opts.dropAllFilenames,
    dropExtensions: opts && opts.dropExtensions || defaultDropExtensions,
    separator: opts && opts.separator,
    userSetRoot: opts && removeLeadingDotSlash(normalisePathSep(opts.root))
  };
};

var getConfigIfNodeTransformable = function getConfigIfNodeTransformable(path, state) {
  // Node matches default placeholder (added in v1).
  if (path.node.name === '__filenamespace') {
    return buildPluginConfig(state.opts);
  } // Node matches custom placeholder (added in v2).
  // Note: always choose 1st one if duplicates exist.


  var placeholder = (state.opts.customPlaceholders || []).filter(function (x) {
    return x && x.placeholder === path.node.name;
  })[0];

  if (placeholder) {
    return buildPluginConfig(placeholder);
  } // Not a transformable node.


  return false;
};

var _default = function _default(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      Identifier: function Identifier(path, state) {
        var pluginConfig = getConfigIfNodeTransformable(path, state);

        if (pluginConfig) {
          // Plugin config
          var dropAllFilenames = pluginConfig.dropAllFilenames,
              dropExtensions = pluginConfig.dropExtensions,
              separator = pluginConfig.separator,
              userSetRoot = pluginConfig.userSetRoot; // Get file paths

          var projectRoot = normalisePathSep(state.cwd);
          var filename = normalisePathSep(state.file.opts.filename);
          var dirname = nodePath.dirname(filename);
          var basename = [nodePath.extname(filename)].concat(dropExtensions).reduce(function (acc, ext) {
            return nodePath.basename(acc, ext);
          }, filename); // Get namespace root directory (defaults to projectRoot)
          // modified via user specified plugin option "root"

          var namespaceRoot = projectRoot;

          if (userSetRoot) {
            // travel up from project dir if userRoot starts with ../, please note
            // that all ../ path segments will be used and all others will be ignored
            // for example:
            // ../ === start from projectFolder
            // ../../ === start from projectParentFolder
            // ../folderA/../ === start from projectParentFolder (folderA is ignored)
            if (userSetRoot.match(/^\.\.\//)) {
              var sourcePathSegments = namespaceRoot.split('/');
              var rootPathSegments = userSetRoot.split('/');
              namespaceRoot = rootPathSegments.filter(function (val) {
                return val.match(/^\.\.$/);
              }).reduce(function (acc) {
                return acc.slice(0, -1);
              }, sourcePathSegments).join('/');
            }
          } // Set namespace root by stripping out unwanted path segments
          // example one:
          // If the babel plugin option root: '../' was set
          // "C:\Users\sam\myProject" would become "myProject"
          // example two:
          // If the babel plugin option root: './src' was set
          // "C:\Users\sam\myProject" would become "myProject/src"


          var filenamespace = dirname.replace(namespaceRoot, '');

          if (userSetRoot && filenamespace.startsWith("/".concat(userSetRoot))) {
            filenamespace = filenamespace.replace(userSetRoot, '');
          } // Add filename to namespace
          // Do not add filename if either:
          // - The `dropAllFilenames` plugin option has been set to true, OR
          // - Filename is "index" as it is meaningless


          var addFilename = !dropAllFilenames && !basename.match(/index/i);

          if (addFilename) {
            filenamespace += "/".concat(basename);
          } // Strip leading path separators


          filenamespace = filenamespace.replace(/^\/+/, ''); // Set custom separator

          if (separator) {
            filenamespace = filenamespace.replace(/\//g, separator);
          } // Replace __filenamespace with the transformed value


          path.replaceWith(t.stringLiteral(filenamespace));
        }
      }
    }
  };
};

exports["default"] = _default;